@@ -4,6 +4,7 @@
 #include <gdk/gdkx.h>
 #include <gdk/gdkkeysyms.h>
 #include "eggcellrendererkeys.h"
+#include "eggaccelerators.h"
 
 #ifndef EGG_COMPILATION
 #ifndef _
@@ -49,9 +50,9 @@ static void egg_cell_renderer_keys_get_size     (GtkCellRenderer *cell,
 enum {
   PROP_0,
 
-  /* FIXME make names consistent with something else */
   PROP_ACCEL_KEY,
-  PROP_ACCEL_MASK
+  PROP_ACCEL_MASK,
+  PROP_ACCEL_MODE
 };
 
 static GtkCellRendererTextClass *parent_class = NULL;
@@ -85,6 +86,7 @@ egg_cell_renderer_keys_get_type (void)
 static void
 egg_cell_renderer_keys_init (EggCellRendererKeys *cell_keys)
 {
+  cell_keys->accel_mode = EGG_CELL_RENDERER_KEYS_MODE_GTK;
 }
 
 /* FIXME setup stuff to generate this */
@@ -152,15 +154,6 @@ egg_cell_renderer_keys_class_init (EggCellRendererKeysClass *cell_keys_class)
    * to match whatever the GTK convention is
    */
   
-  g_object_class_install_property (object_class,
-                                   PROP_ACCEL_MASK,
-                                   g_param_spec_flags ("accel_mask",
-                                                       _("Accelerator modifiers"),
-                                                       _("Accelerator modifiers"),
-                                                       GDK_TYPE_MODIFIER_TYPE,
-                                                       0,
-                                                       G_PARAM_READABLE | G_PARAM_WRITABLE));
-  
   g_object_class_install_property (object_class,
                                    PROP_ACCEL_KEY,
                                    g_param_spec_uint ("accel_key",
@@ -171,6 +164,26 @@ egg_cell_renderer_keys_class_init (EggCellRendererKeysClass *cell_keys_class)
                                                       0,
                                                       G_PARAM_READABLE | G_PARAM_WRITABLE));
 
+  g_object_class_install_property (object_class,
+                                   PROP_ACCEL_MASK,
+                                   g_param_spec_flags ("accel_mask",
+                                                       _("Accelerator modifiers"),
+                                                       _("Accelerator modifiers"),
+                                                       GDK_TYPE_MODIFIER_TYPE,
+                                                       0,
+                                                       G_PARAM_READABLE | G_PARAM_WRITABLE));
+  
+  /* FIXME: Register the enum when moving to GTK+ */
+  g_object_class_install_property (object_class,
+                                   PROP_ACCEL_MODE,
+                                   g_param_spec_int ("accel_mode",
+						     _("Accel Mode"),
+						     _("The type of accelerator."),
+						     0,
+						     2,
+						     0,
+						     G_PARAM_READABLE | G_PARAM_WRITABLE));
+  
   g_signal_new ("keys_edited",
                 EGG_TYPE_CELL_RENDERER_KEYS,
                 G_SIGNAL_RUN_LAST,
@@ -199,13 +212,13 @@ egg_cell_renderer_keys_finalize (GObject *object)
 }
 
 static gchar *
-convert_keysym_state_to_string (guint           keysym,
-                                GdkModifierType state)
+convert_keysym_state_to_string (guint                  keysym,
+                                EggVirtualModifierType mask)
 {
   if (keysym == 0)
     return g_strdup (_("Disabled"));
   else
-    return gtk_accelerator_name (keysym, state);
+    return egg_virtual_accelerator_name (keysym, mask);
 }
 
 static void
@@ -230,6 +243,10 @@ egg_cell_renderer_keys_get_property  (GObject                  *object,
       g_value_set_flags (value, keys->accel_mask);
       break;
 
+    case PROP_ACCEL_MODE:
+      g_value_set_int (value, keys->accel_mode);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
     }
@@ -261,6 +278,10 @@ egg_cell_renderer_keys_set_property  (GObject                  *object,
                                               g_value_get_flags (value));
       break;
 
+    case PROP_ACCEL_MODE:
+      egg_cell_renderer_keys_set_accel_mode (keys, g_value_get_int (value));
+      break;
+      
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
     }
@@ -306,7 +327,7 @@ egg_cell_renderer_keys_get_size (GtkCellRenderer *cell,
   GtkRequisition requisition;
 
   if (keys->sizing_label == NULL)
-    keys->sizing_label = gtk_label_new (_("Type a new shortcut"));
+    keys->sizing_label = gtk_label_new (_("Type a new accelerator, or press Backspace to clear"));
 
   gtk_widget_size_request (keys->sizing_label, &requisition);
   (* GTK_CELL_RENDERER_CLASS (parent_class)->get_size) (cell, widget, cell_area, x_offset, y_offset, width, height);
@@ -317,7 +338,10 @@ egg_cell_renderer_keys_get_size (GtkCellRenderer *cell,
     *height = MAX (*height, requisition.height);
 }
 
-
+/* FIXME: Currently we don't differentiate between a 'bogus' key (like tab in
+ * GTK mode) and a removed key.
+ */
+   
 static gboolean
 grab_key_callback (GtkWidget    *widget,
                    GdkEventKey  *event,
@@ -329,7 +353,9 @@ grab_key_callback (GtkWidget    *widget,
   char *path;
   gboolean edited;
   GdkModifierType consumed_modifiers;  
-	
+  guint upper;
+  GdkModifierType ignored_modifiers;
+  
   keys = EGG_CELL_RENDERER_KEYS (data);
 
   if (is_modifier (event->hardware_keycode))
@@ -344,9 +370,35 @@ grab_key_callback (GtkWidget    *widget,
                                        event->group,
 				       NULL, NULL, NULL, &consumed_modifiers);
 
-  accel_keyval = gdk_keyval_to_lower (event->keyval);
-  accel_mods = event->state & gtk_accelerator_get_default_mod_mask () & ~consumed_modifiers;
+  upper = event->keyval;
+  accel_keyval = gdk_keyval_to_lower (upper);
+  if (accel_keyval == GDK_ISO_Left_Tab) 
+    accel_keyval = GDK_Tab;
+
+
   
+  /* Put shift back if it changed the case of the key, not otherwise.
+   */
+  if (upper != accel_keyval &&
+      (consumed_modifiers & GDK_SHIFT_MASK))
+    {
+      consumed_modifiers &= ~(GDK_SHIFT_MASK);
+    }
+
+  egg_keymap_resolve_virtual_modifiers (gdk_keymap_get_default (),
+                                        EGG_VIRTUAL_NUM_LOCK_MASK |
+                                        EGG_VIRTUAL_SCROLL_LOCK_MASK,
+                                        &ignored_modifiers);
+  
+  /* filter consumed/ignored modifiers */
+
+  if (keys->accel_mode == EGG_CELL_RENDERER_KEYS_MODE_GTK)
+    accel_mods = event->state & ~(consumed_modifiers | ignored_modifiers);
+  else if (keys->accel_mode == EGG_CELL_RENDERER_KEYS_MODE_X)
+    accel_mods = event->state & ~(ignored_modifiers);
+  else
+    g_assert_not_reached ();
+    
   if (accel_mods == 0 && accel_keyval == GDK_Escape)
     goto out; /* cancel */
 
@@ -356,14 +408,19 @@ grab_key_callback (GtkWidget    *widget,
       accel_keyval == GDK_BackSpace)
     accel_keyval = 0;
 
-  if (!gtk_accelerator_valid (accel_keyval, accel_mods))
+  if (keys->accel_mode == EGG_CELL_RENDERER_KEYS_MODE_GTK)
     {
-      accel_keyval = 0;
-      accel_mods = 0;
+      if (!gtk_accelerator_valid (accel_keyval, accel_mods))
+	{
+	  accel_keyval = 0;
+	  accel_mods = 0;
+	}
+
+      /* Remove modifiers like super and hyper, as GTK+ ignores them. */
+      accel_mods = accel_mods & GDK_MODIFIER_MASK;
     }
   
   edited = TRUE;
-  
  out:
   path = g_strdup (g_object_get_data (G_OBJECT (keys->edit_widget),
                                       EGG_CELL_RENDERER_TEXT_PATH));
@@ -503,10 +560,10 @@ egg_cell_renderer_keys_start_editing (GtkCellRenderer      *cell,
   
   if (keys->accel_key != 0)
     gtk_label_set_text (GTK_LABEL (label),
-			_("Type a new shortcut"));
+			_("Type a new accelerator, or press Backspace to clear"));
   else
     gtk_label_set_text (GTK_LABEL (label),
-                        _("Type a new shortcut"));
+                        _("Type a new accelerator"));
 
   gtk_container_add (GTK_CONTAINER (eventbox), label);
   
@@ -519,7 +576,6 @@ egg_cell_renderer_keys_start_editing (GtkCellRenderer      *cell,
                     G_CALLBACK (ungrab_stuff), keys);
   
   keys->edit_key = keys->accel_key;
-  keys->edit_mask = keys->accel_mask;
   
   return GTK_CELL_EDITABLE (keys->edit_widget);
 }
@@ -532,9 +588,9 @@ egg_cell_renderer_keys_set_accelerator (EggCellRendererKeys *keys,
   char *text;
   gboolean changed;
   GtkCellRendererText *celltext;
-  
+
   g_return_if_fail (EGG_IS_CELL_RENDERER_KEYS (keys));
-  
+
   g_object_freeze_notify (G_OBJECT (keys));
 
   changed = FALSE;
@@ -565,7 +621,6 @@ egg_cell_renderer_keys_set_accelerator (EggCellRendererKeys *keys,
   
 }
 
-
 void
 egg_cell_renderer_keys_get_accelerator (EggCellRendererKeys *keys,
                                         guint               *keyval,
@@ -579,3 +634,12 @@ egg_cell_renderer_keys_get_accelerator (EggCellRendererKeys *keys,
   if (mask)
     *mask = keys->accel_mask;
 }
+
+void
+egg_cell_renderer_keys_set_accel_mode (EggCellRendererKeys     *keys,
+				       EggCellRendererKeysMode  accel_mode)
+{
+  g_return_if_fail (EGG_IS_CELL_RENDERER_KEYS (keys));
+  keys->accel_mode = accel_mode;
+  g_object_notify (G_OBJECT (keys), "accel_mode");
+}
